/******************************************************************************

         Copyright (c) 2019 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#ifndef _DSL_CPE_STATUS_PARSER_H
#define _DSL_CPE_STATUS_PARSER_H

#if defined(INCLUDE_DSL_JSON_PARSING) && (INCLUDE_DSL_JSON_PARSING == 1)
#include "dsl_cpe_control.h"

/**
   Definitions for all specified JSON Status File nodes
   (these definitions will be used for different type
   parameters' updates)
*/
typedef enum
{
   DSL_ROOT = 0,
   /* common section */
   DSL_LINE_ENTRIES,/*1*/
   DSL_CHANNEL_ENTRIES,/*2*/
   DSL_BND_GROUP_ENTRIES,/*3*/
   DSL_API_VERSION,/*4*/
   DSL_MEI_VERSION,/*5*/
   DSL_HW_VERSION,/*6*/
   DSL_CHIPSET_TYPE,/*7*/
   /* <dsl.status> section */
   DSL_STATUS,/*8*/
   DSL_STATUS_LINE,/*9*/
      /* <dsl.status.line.0> section */
      DSL_STATUS_LINE_0,/*10*/
      DSL_STATUS_LINE_0_STATUS,/*11*/
      DSL_STATUS_LINE_0_UPSTREAM,/*12*/
      DSL_STATUS_LINE_0_FW_VERSION,/*13*/
      DSL_STATUS_LINE_0_LINK_STATUS,/*14*/
      DSL_STATUS_LINE_0_STD_SUPPORTED,/*15*/
      DSL_STATUS_LINE_0_XTSE,/*16*/
      DSL_STATUS_LINE_0_STD_USED,/*17*/
      DSL_STATUS_LINE_0_XTSE_USED,/*18*/
      DSL_STATUS_LINE_0_ENCODING,/*19*/
      DSL_STATUS_LINE_0_ALLOWED_PROFILES,/*20*/
      DSL_STATUS_LINE_0_CURRENT_PROFILE,/*21*/
      DSL_STATUS_LINE_0_PWR_MGMT_STATE,/*22*/
      DSL_STATUS_LINE_0_SUCC_FAIL_CAUSE,/*23*/
      DSL_STATUS_LINE_0_LAST_STATE_TRANSM,/*24*/
      DSL_STATUS_LINE_0_LAST_STATE_TRANSM_US,/*25*/
      DSL_STATUS_LINE_0_LAST_STATE_TRANSM_DS,/*26*/
      DSL_STATUS_LINE_0_US0_MASK,/*27*/
      DSL_STATUS_LINE_0_TRELLIS,/*28*/
      DSL_STATUS_LINE_0_TRELLIS_US,/*29*/
      DSL_STATUS_LINE_0_TRELLIS_DS,/*30*/
      DSL_STATUS_LINE_0_ACT_SNR_MODE,/*31*/
      DSL_STATUS_LINE_0_ACT_SNR_MODE_US,/*32*/
      DSL_STATUS_LINE_0_ACT_SNR_MODE_DS,/*33*/
      DSL_STATUS_LINE_0_LINE_NR,/*34*/
      DSL_STATUS_LINE_0_MAX_BIT_RATE,/*35*/
      DSL_STATUS_LINE_0_MAX_BIT_RATE_US,/*36*/
      DSL_STATUS_LINE_0_MAX_BIT_RATE_DS,/*37*/
      DSL_STATUS_LINE_0_NOISE_MARGIN,/*38*/
      DSL_STATUS_LINE_0_NOISE_MARGIN_US,/*39*/
      DSL_STATUS_LINE_0_NOISE_MARGIN_DS,/*40*/
      DSL_STATUS_LINE_0_SNR_MPB,/*41*/
      DSL_STATUS_LINE_0_SNR_MPB_US,/*42*/
      DSL_STATUS_LINE_0_SNR_MPB_DS,/*43*/
      DSL_STATUS_LINE_0_POWER,/*44*/
      DSL_STATUS_LINE_0_POWER_US,/*45*/
      DSL_STATUS_LINE_0_POWER_DS,/*46*/
      DSL_STATUS_LINE_0_XTUR_VENDOR,/*47*/
      DSL_STATUS_LINE_0_XTUR_COUNTRY,/*48*/
      DSL_STATUS_LINE_0_XTUR_ANSI_STD,/*49*/
      DSL_STATUS_LINE_0_XTUR_ANSI_REV,/*50*/
      DSL_STATUS_LINE_0_XTUC_VENDOR,/*51*/
      DSL_STATUS_LINE_0_XTUC_COUNTRY,/*52*/
      DSL_STATUS_LINE_0_XTUC_ANSI_STD,/*53*/
      DSL_STATUS_LINE_0_XTUC_ANSI_REV,/*54*/
      DSL_STATUS_LINE_0_UPBOKLER_PB,/*55*/
      DSL_STATUS_LINE_0_RXTHRSH_DS,/*56*/
      DSL_STATUS_LINE_0_ACT_RA_MODE,/*57*/
      DSL_STATUS_LINE_0_ACT_RA_MODE_US,/*58*/
      DSL_STATUS_LINE_0_ACT_RA_MODE_DS,/*59*/
      DSL_STATUS_LINE_0_SNR_MROC,/*60*/
      DSL_STATUS_LINE_0_SNR_MROC_US,/*61*/
      DSL_STATUS_LINE_0_SNR_MROC_DS,/*62*/
      DSL_STATUS_LINE_0_ATTENUATION,/*63*/
      DSL_STATUS_LINE_0_ATTENUATION_US,/*64*/
      DSL_STATUS_LINE_0_ATTENUATION_DS,/*65*/
         /* <dsl.status.line.0.channel> section */
         DSL_STATUS_LINE_0_CHANNEL,/*66*/
         DSL_STATUS_LINE_0_CHANNEL_STATUS,/*67*/
         DSL_STATUS_LINE_0_CHANNEL_LINKENC_SUPP,/*68*/
         DSL_STATUS_LINE_0_CHANNEL_LINKENC_USED,/*69*/
         DSL_STATUS_LINE_0_CHANNEL_LPATH,/*70*/
         DSL_STATUS_LINE_0_CHANNEL_INTLVDEPTH,/*71*/
         DSL_STATUS_LINE_0_CHANNEL_INTLVBLOCK,/*72*/
         DSL_STATUS_LINE_0_CHANNEL_ACT_INTERL_DELAY,/*73*/
         DSL_STATUS_LINE_0_CHANNEL_ACTINP,/*74*/
         DSL_STATUS_LINE_0_CHANNEL_INPREPORT,/*75*/
         DSL_STATUS_LINE_0_CHANNEL_NFEC,/*76*/
         DSL_STATUS_LINE_0_CHANNEL_RFEC,/*77*/
         DSL_STATUS_LINE_0_CHANNEL_LSYMB,/*78*/
         DSL_STATUS_LINE_0_CHANNEL_DATARATE,/*79*/
         DSL_STATUS_LINE_0_CHANNEL_DATARATE_US,/*80*/
         DSL_STATUS_LINE_0_CHANNEL_DATARATE_DS,/*81*/
         DSL_STATUS_LINE_0_CHANNEL_ACTNDR,/*82*/
         DSL_STATUS_LINE_0_CHANNEL_ACTNDR_US,/*83*/
         DSL_STATUS_LINE_0_CHANNEL_ACTNDR_DS,/*84*/
         DSL_STATUS_LINE_0_CHANNEL_ACTINPREIN,/*85*/
         DSL_STATUS_LINE_0_CHANNEL_ACTINPREIN_US,/*86*/
         DSL_STATUS_LINE_0_CHANNEL_ACTINPREIN_DS,/*87*/
      /* <dsl.status.line.1> section */
      DSL_STATUS_LINE_1,/*88*/
      DSL_STATUS_LINE_1_STATUS,/*89*/
      DSL_STATUS_LINE_1_UPSTREAM,/*90*/
      DSL_STATUS_LINE_1_FW_VERSION,/*91*/
      DSL_STATUS_LINE_1_LINK_STATUS,/*92*/
      DSL_STATUS_LINE_1_STD_SUPPORTED,/*93*/
      DSL_STATUS_LINE_1_XTSE,/*94*/
      DSL_STATUS_LINE_1_STD_USED,/*95*/
      DSL_STATUS_LINE_1_XTSE_USED,/*96*/
      DSL_STATUS_LINE_1_ENCODING,/*97*/
      DSL_STATUS_LINE_1_ALLOWED_PROFILES,/*98*/
      DSL_STATUS_LINE_1_CURRENT_PROFILE,/*99*/
      DSL_STATUS_LINE_1_PWR_MGMT_STATE,/*100*/
      DSL_STATUS_LINE_1_SUCC_FAIL_CAUSE,/*101*/
      DSL_STATUS_LINE_1_LAST_STATE_TRANSM,/*102*/
      DSL_STATUS_LINE_1_LAST_STATE_TRANSM_US,/*103*/
      DSL_STATUS_LINE_1_LAST_STATE_TRANSM_DS,/*104*/
      DSL_STATUS_LINE_1_US0_MASK,/*105*/
      DSL_STATUS_LINE_1_TRELLIS,/*106*/
      DSL_STATUS_LINE_1_TRELLIS_US,/*107*/
      DSL_STATUS_LINE_1_TRELLIS_DS,/*108*/
      DSL_STATUS_LINE_1_ACT_SNR_MODE,/*109*/
      DSL_STATUS_LINE_1_ACT_SNR_MODE_US,/*110*/
      DSL_STATUS_LINE_1_ACT_SNR_MODE_DS,/*111*/
      DSL_STATUS_LINE_1_LINE_NR,/*112*/
      DSL_STATUS_LINE_1_MAX_BIT_RATE,/*113*/
      DSL_STATUS_LINE_1_MAX_BIT_RATE_US,/*114*/
      DSL_STATUS_LINE_1_MAX_BIT_RATE_DS,/*115*/
      DSL_STATUS_LINE_1_NOISE_MARGIN,/*116*/
      DSL_STATUS_LINE_1_NOISE_MARGIN_US,/*117*/
      DSL_STATUS_LINE_1_NOISE_MARGIN_DS,/*118*/
      DSL_STATUS_LINE_1_SNR_MPB,/*119*/
      DSL_STATUS_LINE_1_SNR_MPB_US,/*120*/
      DSL_STATUS_LINE_1_SNR_MPB_DS,/*121*/
      DSL_STATUS_LINE_1_POWER,/*122*/
      DSL_STATUS_LINE_1_POWER_US,/*123*/
      DSL_STATUS_LINE_1_POWER_DS,/*124*/
      DSL_STATUS_LINE_1_XTUR_VENDOR,/*125*/
      DSL_STATUS_LINE_1_XTUR_COUNTRY,/*126*/
      DSL_STATUS_LINE_1_XTUR_ANSI_STD,/*127*/
      DSL_STATUS_LINE_1_XTUR_ANSI_REV,/*128*/
      DSL_STATUS_LINE_1_XTUC_VENDOR,/*129*/
      DSL_STATUS_LINE_1_XTUC_COUNTRY,/*130*/
      DSL_STATUS_LINE_1_XTUC_ANSI_STD,/*131*/
      DSL_STATUS_LINE_1_XTUC_ANSI_REV,/*132*/
      DSL_STATUS_LINE_1_UPBOKLER_PB,/*133*/
      DSL_STATUS_LINE_1_RXTHRSH_DS,/*134*/
      DSL_STATUS_LINE_1_ACT_RA_MODE,/*135*/
      DSL_STATUS_LINE_1_ACT_RA_MODE_US,/*136*/
      DSL_STATUS_LINE_1_ACT_RA_MODE_DS,/*137*/
      DSL_STATUS_LINE_1_SNR_MROC,/*138*/
      DSL_STATUS_LINE_1_SNR_MROC_US,/*139*/
      DSL_STATUS_LINE_1_SNR_MROC_DS,/*140*/
      DSL_STATUS_LINE_1_ATTENUATION,/*141*/
      DSL_STATUS_LINE_1_ATTENUATION_US,/*142*/
      DSL_STATUS_LINE_1_ATTENUATION_DS,/*143*/
         /* <dsl.status.line.1.channel> section */
         DSL_STATUS_LINE_1_CHANNEL,/*144*/
         DSL_STATUS_LINE_1_CHANNEL_STATUS,/*145*/
         DSL_STATUS_LINE_1_CHANNEL_LINKENC_SUPP,/*146*/
         DSL_STATUS_LINE_1_CHANNEL_LINKENC_USED,/*147*/
         DSL_STATUS_LINE_1_CHANNEL_LPATH,/*148*/
         DSL_STATUS_LINE_1_CHANNEL_INTLVDEPTH,/*149*/
         DSL_STATUS_LINE_1_CHANNEL_INTLVBLOCK,/*150*/
         DSL_STATUS_LINE_1_CHANNEL_ACT_INTERL_DELAY,/*151*/
         DSL_STATUS_LINE_1_CHANNEL_ACTINP,/*152*/
         DSL_STATUS_LINE_1_CHANNEL_INPREPORT,/*153*/
         DSL_STATUS_LINE_1_CHANNEL_NFEC,/*154*/
         DSL_STATUS_LINE_1_CHANNEL_RFEC,/*155*/
         DSL_STATUS_LINE_1_CHANNEL_LSYMB,/*156*/
         DSL_STATUS_LINE_1_CHANNEL_DATARATE,/*157*/
         DSL_STATUS_LINE_1_CHANNEL_DATARATE_US,/*158*/
         DSL_STATUS_LINE_1_CHANNEL_DATARATE_DS,/*159*/
         DSL_STATUS_LINE_1_CHANNEL_ACTNDR,/*160*/
         DSL_STATUS_LINE_1_CHANNEL_ACTNDR_US,/*161*/
         DSL_STATUS_LINE_1_CHANNEL_ACTNDR_DS,/*162*/
         DSL_STATUS_LINE_1_CHANNEL_ACTINPREIN,/*163*/
         DSL_STATUS_LINE_1_CHANNEL_ACTINPREIN_US,/*164*/
         DSL_STATUS_LINE_1_CHANNEL_ACTINPREIN_DS,/*165*/
   /* <dsl.stats> section */
   DSL_STATS,/*166*/
   DSL_STATS_LINE,/*167*/
      /* <dsl.stats.line.0> section */
      DSL_STATS_LINE_0,/*168*/
      DSL_STATS_LINE_0_TOTAL_START,/*169*/
      DSL_STATS_LINE_0_SHOWTIME_START,/*170*/
      DSL_STATS_LINE_0_LAST_SHOWTIME_START,/*171*/
      DSL_STATS_LINE_0_CURRENT_DAY_START,/*172*/
      DSL_STATS_LINE_0_QUARTER_HOUR_START,/*173*/
      DSL_STATS_LINE_0_TOTAL,/*174*/
      DSL_STATS_LINE_0_TOTAL_ERR_SECS,/*175*/
      DSL_STATS_LINE_0_TOTAL_SEV_ERR_SECS,/*176*/
      DSL_STATS_LINE_0_SHOWTIME,/*177*/
      DSL_STATS_LINE_0_SHOWTIME_ERR_SECS,/*178*/
      DSL_STATS_LINE_0_SHOWTIME_SEV_ERR_SECS,/*179*/
      DSL_STATS_LINE_0_LAST_SHOWTIME,/*180*/
      DSL_STATS_LINE_0_LAST_SHOWTIME_ERR_SECS,/*181*/
      DSL_STATS_LINE_0_LAST_SHOWTIME_SEV_ERR_SECS,/*182*/
      DSL_STATS_LINE_0_CURRENT_DAY,/*183*/
      DSL_STATS_LINE_0_CURRENT_DAY_ERR_SECS,/*184*/
      DSL_STATS_LINE_0_CURRENT_DAY_SEV_ERR_SECS,/*185*/
      DSL_STATS_LINE_0_QUARTER_HOUR,/*186*/
      DSL_STATS_LINE_0_QUARTER_HOUR_ERR_SECS,/*187*/
      DSL_STATS_LINE_0_QUARTER_HOUR_SEV_ERR_SECS,/*188*/
         /* <dsl.stats.line.0.channel> section */
         DSL_STATS_LINE_0_CHANNEL,/*189*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_START,/*190*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_START,/*191*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_START,/*192*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_START,/*193*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_START,/*194*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL,/*195*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUR_FEC_ERRORS,/*196*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUR_CRC_ERRORS,/*197*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUC_FEC_ERRORS,/*198*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUC_CRC_ERRORS,/*199*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUR_HEC_ERRORS,/*200*/
         DSL_STATS_LINE_0_CHANNEL_TOTAL_XTUC_HEC_ERRORS,/*201*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME,/*202*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUR_FEC_ERRORS,/*203*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUR_CRC_ERRORS,/*204*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUC_FEC_ERRORS,/*205*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUC_CRC_ERRORS,/*206*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUR_HEC_ERRORS,/*207*/
         DSL_STATS_LINE_0_CHANNEL_SHOWTIME_XTUC_HEC_ERRORS,/*208*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME,/*209*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUR_FEC_ERRORS,/*210*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUR_CRC_ERRORS,/*211*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUC_FEC_ERRORS,/*212*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUC_CRC_ERRORS,/*213*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUR_HEC_ERRORS,/*214*/
         DSL_STATS_LINE_0_CHANNEL_LAST_SHOWTIME_XTUC_HEC_ERRORS,/*215*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY,/*216*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUR_FEC_ERRORS,/*217*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUR_CRC_ERRORS,/*218*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUC_FEC_ERRORS,/*219*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUC_CRC_ERRORS,/*220*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUR_HEC_ERRORS,/*221*/
         DSL_STATS_LINE_0_CHANNEL_CURRENT_DAY_XTUC_HEC_ERRORS,/*222*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR,/*223*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUR_FEC_ERRORS,/*224*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUR_CRC_ERRORS,/*225*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUC_FEC_ERRORS,/*226*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUC_CRC_ERRORS,/*227*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUR_HEC_ERRORS,/*228*/
         DSL_STATS_LINE_0_CHANNEL_QUARTER_HOUR_XTUC_HEC_ERRORS,/*229*/
      /* <dsl.stats.line.1> section */
      DSL_STATS_LINE_1,/*230*/
      DSL_STATS_LINE_1_TOTAL_START,/*231*/
      DSL_STATS_LINE_1_SHOWTIME_START,/*232*/
      DSL_STATS_LINE_1_LAST_SHOWTIME_START,/*233*/
      DSL_STATS_LINE_1_CURRENT_DAY_START,/*234*/
      DSL_STATS_LINE_1_QUARTER_HOUR_START,/*235*/
      DSL_STATS_LINE_1_TOTAL,/*236*/
      DSL_STATS_LINE_1_TOTAL_ERR_SECS,/*237*/
      DSL_STATS_LINE_1_TOTAL_SEV_ERR_SECS,/*238*/
      DSL_STATS_LINE_1_SHOWTIME,/*239*/
      DSL_STATS_LINE_1_SHOWTIME_ERR_SECS,/*240*/
      DSL_STATS_LINE_1_SHOWTIME_SEV_ERR_SECS,/*241*/
      DSL_STATS_LINE_1_LAST_SHOWTIME,/*242*/
      DSL_STATS_LINE_1_LAST_SHOWTIME_ERR_SECS,/*243*/
      DSL_STATS_LINE_1_LAST_SHOWTIME_SEV_ERR_SECS,/*244*/
      DSL_STATS_LINE_1_CURRENT_DAY,/*245*/
      DSL_STATS_LINE_1_CURRENT_DAY_ERR_SECS,/*246*/
      DSL_STATS_LINE_1_CURRENT_DAY_SEV_ERR_SECS,/*247*/
      DSL_STATS_LINE_1_QUARTER_HOUR,/*248*/
      DSL_STATS_LINE_1_QUARTER_HOUR_ERR_SECS,/*249*/
      DSL_STATS_LINE_1_QUARTER_HOUR_SEV_ERR_SECS,/*250*/
         /* <dsl.stats.line.1.channel> section */
         DSL_STATS_LINE_1_CHANNEL,/*251*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_START,/*252*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_START,/*253*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_START,/*254*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_START,/*255*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_START,/*256*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL,/*257*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUR_FEC_ERRORS,/*258*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUR_CRC_ERRORS,/*259*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUC_FEC_ERRORS,/*260*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUC_CRC_ERRORS,/*261*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUR_HEC_ERRORS,/*262*/
         DSL_STATS_LINE_1_CHANNEL_TOTAL_XTUC_HEC_ERRORS,/*263*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME,/*264*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUR_FEC_ERRORS,/*265*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUR_CRC_ERRORS,/*266*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUC_FEC_ERRORS,/*267*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUC_CRC_ERRORS,/*268*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUR_HEC_ERRORS,/*269*/
         DSL_STATS_LINE_1_CHANNEL_SHOWTIME_XTUC_HEC_ERRORS,/*270*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME,/*271*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUR_FEC_ERRORS,/*272*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUR_CRC_ERRORS,/*273*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUC_FEC_ERRORS,/*274*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUC_CRC_ERRORS,/*275*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUR_HEC_ERRORS,/*276*/
         DSL_STATS_LINE_1_CHANNEL_LAST_SHOWTIME_XTUC_HEC_ERRORS,/*277*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY,/*278*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUR_FEC_ERRORS,/*279*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUR_CRC_ERRORS,/*280*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUC_FEC_ERRORS,/*281*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUC_CRC_ERRORS,/*282*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUR_HEC_ERRORS,/*283*/
         DSL_STATS_LINE_1_CHANNEL_CURRENT_DAY_XTUC_HEC_ERRORS,/*284*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR,/*285*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUR_FEC_ERRORS,/*286*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUR_CRC_ERRORS,/*287*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUC_FEC_ERRORS,/*288*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUC_CRC_ERRORS,/*289*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUR_HEC_ERRORS,/*290*/
         DSL_STATS_LINE_1_CHANNEL_QUARTER_HOUR_XTUC_HEC_ERRORS/*291*/
} DSL_CPE_StatusNodeIdx_t;

/**
   This function creates the initial Status File
   and updates the init parameters values

   \param pContext   - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_CPE_StatusFileInitialize(
   DSL_CPE_Control_Context_t *pContext);

/**
   This function updates Status File parameters
   triggered by events

   \param pContext   - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_CPE_StatusFileEvtUpdate(
   DSL_CPE_Control_Context_t *pContext,
   const DSL_CPE_StatusNodeIdx_t eNodeIdx);

/**
   This function updates Status File parameters
   triggered by the polling timer

   \param pContext   - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_CPE_StatusFilePollUpdate(
   DSL_CPE_Control_Context_t *pContext,
   const DSL_CPE_StatusNodeIdx_t ePollSectionIdx);

#endif /* defined(INCLUDE_DSL_JSON_PARSING) && (INCLUDE_DSL_JSON_PARSING == 1) */

#endif /* _LIB_DSL_STATUS_PARSER_H */
